@@ -54,6 +54,58 @@ def generate_incident_field(maskfunction,alpha,f,divisions_phi,divisions_theta,g
     
     return ex_lens,ey_lens
 
+def generate_rotated_incident_field(maskfunction,alpha,f,divisions_phi,divisions_theta,gamma,beta,w0,I0,wavelength):
+    '''
+    Generate a matrix for the field X and Y direction of the incident field on the lens evaluated at phi-180º for ex_lens and at phi-270º for ey_lens, given the respective maskfunction
+    
+    Args:        
+        :maskfunction (function): Analytical function that defines the phase mask, must be a function of the 5 internal variables: rho, phi, w0, f and k, with:
+            
+            rho: Radial coordinate from 0 to the aperture radius of the objective.
+            
+            phi: Azimutal coordinate from 0 to 2pi.
+            
+            w0: Radius of the incident gaussian beam.
+            
+            f: Focal distane of the objective lens (mm)
+            
+            k: Wavenumber in the objective lens medium (mm)
+            
+            The real part defines the amplitude of the incident field
+    
+        :divisions_phi,divisions_theta: Number of divisions in the phi and theta coordinates to use the 2D integration for the calculation of the focused field
+ 
+        The rest of the parameters are specified in sim
+
+    Returns:
+        :arrays: ex_lens,ey_lens
+        
+    This arrays have the value of the amplitude of the incident field for each value of theta and phi: ex_lens[phi_position,theta_position]
+    for phi_position a value in np.linspace(0,2*np.pi,divisions_phi) and theta_position a value in np.linspace(0,alpha,divisions_theta) 
+    '''
+    wavelength/=10**6#pasage from nm to mm
+    k=2*np.pi/wavelength #k is given in nm, the same as wavelength
+    ex_lens=np.zeros((divisions_phi,divisions_theta),dtype=complex)
+    ey_lens=np.zeros((divisions_phi,divisions_theta),dtype=complex)
+
+    theta_values=np.linspace(0,alpha,divisions_theta)  #divisions of theta in which the trapezoidal 2D integration is done
+    rho_values=np.sin(theta_values)*f              #given by the sine's law
+    phi_values_x=np.linspace(np.pi,3*np.pi,divisions_phi)   #divisions of phi in which the trapezoidal 2D integration is done
+    phi_values_y=np.linspace(3*np.pi/2,7*np.pi/2,divisions_phi)   #divisions of phi in which the trapezoidal 2D integration is done
+    for i,phi in enumerate(phi_values_x):
+        for j,rho in enumerate(rho_values):
+            phase=maskfunction(rho,phi,w0,f,k)
+            ex_lens[i,j]=phase
+    for i,phi in enumerate(phi_values_y):
+        for j,rho in enumerate(rho_values):
+            phase=maskfunction(rho,phi,w0,f,k)
+            ey_lens[i,j]=phase
+    ex_lens*=np.cos(gamma*np.pi/180)*I0**0.5
+    ey_lens*=np.sin(gamma*np.pi/180)*np.exp(1j*beta*np.pi/180)*I0**0.5
+    
+    return ex_lens,ey_lens
+
+
 def plot_in_cartesian(Ex,Ey,r_range,alpha,f,figure_name):
     '''
     Plot the fields Ex and Ey, who are described in the same coordinates as ex_lens and ey_lens. To do so the field in the closest cartesian coordinates for each position is calculated
@@ -235,7 +287,7 @@ def custom_mask_objective_field(h,gamma,beta,divisions_theta,divisions_phi,N_rho
     return Ex,Ey,I_cartesian,Ex_cartesian,Ey_cartesian
 
 
-def custom_mask_focus_field_XY(ex_lens,ey_lens,alpha,h,wavelength,zp0,resolution_x,divisions_theta,divisions_phi,x_range,countdown=True,x0=0,y0=0):
+def custom_mask_focus_field_XY(ex_lens,ey_lens,alpha,h,wavelength,zp0,resolution_x,divisions_theta,divisions_phi,x_range,countdown=True,x0=0,y0=0,field_is_already_rotated=False):
     '''
     2D integration to calculate the field focused by a high aperture lens on the XY plane
     
@@ -253,6 +305,8 @@ def custom_mask_focus_field_XY(ex_lens,ey_lens,alpha,h,wavelength,zp0,resolution
         :x0, y0: Used for centering the XY field at an x0, y0 position (nm)
 
         :countdown (bool): True for shoing a progress bar with the time elapsed and expected to finish the calculation, True not recomended if this function is to be used many times
+        
+        :field_is_already_rotated (bool): Internal variable used to tell if the field is already evaluated at phi-180º and phi-270º
 
         The rest of the parameters are specified in sim
     
@@ -286,8 +340,9 @@ def custom_mask_focus_field_XY(ex_lens,ey_lens,alpha,h,wavelength,zp0,resolution
     #To use the incident field as the function to be integrated in the equations deried from born and wolf, the incident field must be evaluated at phi-pi for the X component and at phi-3*pi/2 for the Y component. 
     #The motives for this correspond to a difference in the coordinate system used when deriving the equations, where the z versor points in the oposite direction
     #This fact is not described in the paper since i am not 100% sure this is the motive, but the rotation is necesary to obtain the needed result in all my tests with custom masks
-    ex_lens=rotate_180º(ex_lens)
-    ey_lens=rotate_270º(ey_lens)
+    if not field_is_already_rotated:
+        ex_lens=rotate_180º(ex_lens)
+        ey_lens=rotate_270º(ey_lens)
     
     '''
     # the functions i am going to integrate are:
@@ -381,7 +436,7 @@ def custom_mask_focus_field_XY(ex_lens,ey_lens,alpha,h,wavelength,zp0,resolution
     
     return ex,ey,ez
 
-def custom_mask_focus_field_XZ_XY(ex_lens,ey_lens,alpha,h,wavelength,z_range,resolution_z,zp0,resolution_x,divisions_theta,divisions_phi,x_range,x0=0,y0=0,plot_plane='X'):
+def custom_mask_focus_field_XZ_XY(ex_lens,ey_lens,alpha,h,wavelength,z_range,resolution_z,zp0,resolution_x,divisions_theta,divisions_phi,x_range,x0=0,y0=0,plot_plane='X',field_is_already_rotated=False):
     '''
     2D integration to calculate the field focused by a high aperture lens on the XY and XZ plane
     
@@ -401,6 +456,8 @@ def custom_mask_focus_field_XZ_XY(ex_lens,ey_lens,alpha,h,wavelength,z_range,res
         :x0, y0: Used for centering the XY field at an x0, y0 position (nm)
         
         :plot_plane (string): Available values: 'X' or 'Y', select to plot the ZX or the ZY plane respectivelly
+        
+        :field_is_already_rotated (bool): Internal variable used to tell if the field is already evaluated at phi-180º and phi-270º
 
         The rest of the parameters are specified in sim
     
@@ -416,7 +473,7 @@ def custom_mask_focus_field_XZ_XY(ex_lens,ey_lens,alpha,h,wavelength,z_range,res
         The XZ plane is given by y=0 and the XZ plane by z=zp0 
     '''
     #XY plane: 
-    Ex_XY,Ey_XY,Ez_XY=custom_mask_focus_field_XY(ex_lens,ey_lens,alpha,h,wavelength,zp0,resolution_x,divisions_theta,divisions_phi,x_range,countdown=True,x0=x0,y0=y0)
+    Ex_XY,Ey_XY,Ez_XY=custom_mask_focus_field_XY(ex_lens,ey_lens,alpha,h,wavelength,zp0,resolution_x,divisions_theta,divisions_phi,x_range,countdown=True,x0=x0,y0=y0,field_is_already_rotated=field_is_already_rotated)
     
     #XZ plane:
     if int(resolution_z%2)==0:
@@ -443,8 +500,9 @@ def custom_mask_focus_field_XZ_XY(ex_lens,ey_lens,alpha,h,wavelength,z_range,res
     #To use the incident field as the function to be integrated in the equations deried from born and wolf, the incident field must be evaluated at phi-pi for the X component and at phi-3*pi/2 for the Y component. 
     #The motives for this correspond to a difference in the coordinate system used when deriving the equations, where the z versor points in the oposite direction
     #This fact is not described in the paper since i am not 100% sure this is the motive, but the rotation is necesary to obtain the needed result in all my tests with custom masks
-    ex_lens=rotate_180º(ex_lens)
-    ey_lens=rotate_270º(ey_lens)
+    if not field_is_already_rotated:
+        ex_lens=rotate_180º(ex_lens)
+        ey_lens=rotate_270º(ey_lens)
 
     '''
     # the functions i am going to integrate are:
